using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using com.ibm.gre;

namespace SampleApp
{
	/// <summary>
	/// The ComponentForm is a very simple example of
	/// displaying File Plan Component data for viewing
	/// adding and editing.
	/// It demonstrates how to use the IRMFields object
	/// to iterate through a Fields Collection to display values
	/// and to retrieve user input for a File Plan Component. 
	/// </summary>
	public class ComponentForm : System.Windows.Forms.Form
	{
		/// <summary>
		/// Constructor:
		/// </summary>
		/// <param name="session">The current ClientSession/</param>
		/// <param name="irm">The IRMComponent from the selected
		/// Tree View node or List View item that the File Plan Component
		/// information will be displayed for.</param>
		/// <param name="parent">A reference to the Form1 object used for logging.</param>
		/// <param name="fields">An IRMFields object encapsulating the File Plan Component
		/// information that is to be displayed.</param>
		/// <param name="action">The action to be performed. Can be one of ADD, EDIT or VIEW.</param>
		public ComponentForm(ClientSession session, IRMComponent irm, Control parent, IRMFields fields, byte action)
		{
			// Required for Windows Form Designer support
			InitializeComponent();
			m_session = session;
			m_irm = irm;
			m_parent = parent;
			util = new Util(this,parent);
			m_fields = fields;
			m_nAction = action;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.btnCancel = new System.Windows.Forms.Button();
			this.btnOk = new System.Windows.Forms.Button();
			this.label1 = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// btnCancel
			// 
			this.btnCancel.Location = new System.Drawing.Point(424, 40);
			this.btnCancel.Name = "btnCancel";
			this.btnCancel.TabIndex = 9;
			this.btnCancel.Text = "Cancel";
			this.btnCancel.Click += new System.EventHandler(this.btnCancel_Click);
			// 
			// btnOk
			// 
			this.btnOk.Location = new System.Drawing.Point(424, 8);
			this.btnOk.Name = "btnOk";
			this.btnOk.TabIndex = 8;
			this.btnOk.Text = "Okay";
			this.btnOk.Click += new System.EventHandler(this.btnOk_Click);
			// 
			// label1
			// 
			this.label1.BackColor = System.Drawing.Color.Gold;
			this.label1.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.label1.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label1.Location = new System.Drawing.Point(8, 8);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(408, 56);
			this.label1.TabIndex = 10;
			// 
			// ComponentForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.AutoScroll = true;
			this.ClientSize = new System.Drawing.Size(504, 77);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.btnCancel);
			this.Controls.Add(this.btnOk);
			this.Name = "ComponentForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "ComponentForm";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.ComponentForm_Closing);
			this.Load += new System.EventHandler(this.ComponentForm_Load);
			this.ResumeLayout(false);

		}
		#endregion

		#region events
		private void ComponentForm_Load(object sender, System.EventArgs e)
		{
			const int space = 6;
			const int tabWidth = 5;
			const int controlWidth = 200;
			const int topSpace = 5;
			const int fudgeFactor = 48;

			int x = label1.Left; //.Location.X;
			int y = label1.Top + label1.Height + topSpace;

			int displayCount = m_fields.DisplayCount; //Only count visible fields
			util.log("IRMFields.DisplayCount = " + displayCount);

			Label tempLbl = new Label();
			tempLbl.AutoSize = true;
			tempLbl.Text = m_fields.MaxCaption;
			int height = tempLbl.Height;

			int captionWidth = tempLbl.Width;
			if(captionWidth < 200)
			{
				captionWidth = 200;
			}
			
			int workingHeight = System.Windows.Forms.SystemInformation.WorkingArea.Height;
			int workingWidth = System.Windows.Forms.SystemInformation.WorkingArea.Width;

			int formHeight = label1.Height + fudgeFactor + displayCount * (height + space);
			if(formHeight > workingHeight)
			{
				formHeight = workingHeight - (workingHeight / 8);
			}
			
			//m_fields.getFieldByFlPlnCmpntAttrDefColNm("FlPlnCmpntNm");

			switch(m_nAction)
			{
				case ComponentForm.ADD:
					label1.Text = "Add Component to " + m_irm.FlPlnCmpntNm;
					break;
				case ComponentForm.EDIT:
					label1.Text = "Edit Component " + m_irm.FlPlnCmpntNm;
					break;
				case ComponentForm.VIEW:
					btnCancel.Visible = false;
					label1.Text = "View Component " + m_irm.FlPlnCmpntNm;
					break;
			}
			int delta = 0;
			if(captionWidth > 200)
			{
				delta = captionWidth - 200;
				this.Width = this.Width + delta;
				label1.Width = label1.Width + delta;
				btnOk.Left = btnOk.Left + delta;
				btnCancel.Left = btnCancel.Left + delta;
			}
			this.Height = formHeight; 
			this.Left = (workingWidth - this.Width)/2;
			this.Top = (workingHeight - formHeight)/2;

			for (int i = 0; i < m_fields.FieldsCount; i++)
			{
				IRMField field = m_fields.getField(i);
				if(field.isVisible) //Only show fields that have been spefiied visible
				{
					util.log("Getting IRMField object using IRMFields.getField(" + i + ")");

					Label lbl = new Label();
					lbl.Location = new Point(x + tabWidth,y);
					lbl.Width = captionWidth;
					lbl.Text = field.Caption;
					this.Controls.Add (lbl);

					switch(field.ControlType)
					{
							//TODO many more cases could be handled
						case IRMConstants.ProfileControlType.SINGLELINE:
							TextBox txtBox = new TextBox();
							txtBox.Location = new Point(x + lbl.Width + (2*tabWidth) ,y);
							txtBox.Width = controlWidth;
							txtBox.Tag = field.FlPlnCmpntAttrDefColNm;
							util.log("Add a TextBox at (" + x + "," + y + ") with Key value: " + field.FlPlnCmpntAttrDefColNm);
							txtBox.Text = field.Value;
							this.Controls.Add (txtBox);
							break;
						case IRMConstants.ProfileControlType.DROPLIST:
							ComboBox droplist = new ComboBox();
							droplist.DropDownStyle = ComboBoxStyle.DropDownList;
							droplist.Location = new Point(x + lbl.Width + (2*tabWidth),y);
							droplist.Width = controlWidth;
							droplist.Tag = field.FlPlnCmpntAttrDefColNm;
							util.log("Add a DropList with Key value: " + field.FlPlnCmpntAttrDefColNm);

							//TODO expand this for all types
							/*
							if(field.PickListId != 0)
							{
								util.log("Populate the DropList with values from PickList: " + line.PickListId);

								string xml = m_irmQuery.ClientSession.getComponentDesignController().getPickListEntryList(line.PickListId,0,0);
								
								util.log("Call the ComponentDesignController method getPickListEntryList for XML values\r\n" + xml);

								XmlDocument doc = new XmlDocument();
								doc.LoadXml(xml);
								XmlElement root = doc.DocumentElement;
								XmlNodeList items = root.ChildNodes;
								for(int j = 0; j < items.Count; j++)
								{
									string strPckLstEntVal = items.Item(j).Attributes.GetNamedItem("PckLstEntVal").Value;
									droplist.Items.Add(strPckLstEntVal);
								}
							}
							*/
							this.Controls.Add(droplist);
							break;
						case IRMConstants.ProfileControlType.COMBOBOX:
							ComboBox combo = new ComboBox();
							combo.Location = new Point(x + lbl.Width + (2*tabWidth),y);
							combo.Width = controlWidth;
							combo.Tag = field.FlPlnCmpntAttrDefColNm;

							util.log("Add a ComboBox with Key value: " + field.FlPlnCmpntAttrDefColNm);

							this.Controls.Add(combo);
							break;
						default:
							//TODO determine appropriate action here.
							break;

					}

					y += height + space;
				}
			}

		}

		private void ComponentForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			if(!valid)
			{
				e.Cancel = true;
				valid = true;
			}
		}

		private void btnCancel_Click(object sender, System.EventArgs e)
		{
			//valid = false;
			DialogResult = DialogResult.Cancel;
		}

		private void btnOk_Click(object sender, System.EventArgs e)
		{
			try
			{
				for(int i = 0; i < this.Controls.Count; i++)
				{
					if(i == 0)
					{
						util.log("Set the modified Query values");
					}
					string strFlPlnCmpntAttrDefColNm = (string)this.Controls[i].Tag;
					if(strFlPlnCmpntAttrDefColNm != null)
					{
						string val = this.Controls[i].Text;
						//m_irmQuery.setFieldValue returns false if the value is not of the right type
						//or cannot be found
						//TODO logic should be improved
 
						util.log("Call IRMQuery.setFieldValue(" + strFlPlnCmpntAttrDefColNm + ", " + val + ")");
						if(!m_fields.setFieldByFlPlnCmpntAttrDefColNm(strFlPlnCmpntAttrDefColNm,val))
						{
							MessageBox.Show("Invaled value for " + this.Controls[i].Text);
							valid = false;
							DialogResult = DialogResult.Abort;
							return;
						}
					}
				}
				switch(m_nAction)
				{
					case ADD:

						util.log("m_fields.FlPlnCmpntDefID: " + m_fields.FlPlnCmpntDefID);
						util.log(m_fields.getXMLForAdd());

						//TODO move this logic out of the UI component
						if(m_fields.FlPlnCmpntDefID == (long)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_VIEW)
						{
							m_nFlPlnCmpntID = m_session.getTemplateController().addNewSystemComponent
								(
									m_fields.getXMLForAdd(),
									m_fields.FlPlnCmpntDefID,
									0
								);
						}
						else
						{
							//TODO Add logic for default ID
							m_nFlPlnCmpntID = m_session.getTemplateController().addNewFilePlanComponent
								(
								m_fields.getXMLForAdd(),
								m_fields.FlPlnCmpntDefID,
								m_irm.FlPlnCmpntID,
								0
								);
						}
						break;
					case EDIT:
						string taskId = m_session.getFilePlanComponentController().updateFilePlanComponent(m_irm.FlPlnCmpntID,m_fields.getXMLForAdd(),false);
						//TODO this is a cheap trick for checking taskId (Always assumes non batch string will be  <TaskID /> exactly) 
						if(taskId != "<TaskID/>")
						{
							MessageBox.Show("The update was executed in BATCH mode check the Records Manager Administrator to see the results of this operation");
						}
						break;
					case VIEW:
						break;
				}
			}
			catch(System.Net.WebException we)
			{
				MessageBox.Show(we.Message);
				DialogResult = DialogResult.Abort;
				valid = false;
				return;
			}
			catch(System.Web.Services.Protocols.SoapException s)
			{
				MessageBox.Show(s.Message);
				DialogResult = DialogResult.Abort;
				valid = false;
				return;
			}
			valid = true;
			DialogResult = DialogResult.OK;

		}
		#endregion

		public byte Action
		{
			get
			{
				return m_nAction;
			}
			set
			{
				m_nAction = value;
			}
		}

		public long FlPlnCmpntID
		{
			get
			{
				return m_nFlPlnCmpntID;
			}
		}

		#region MemberVariables
		public const byte ADD = 1;
		public const byte EDIT = 2;
		public const byte VIEW = 3;

		private byte m_nAction;

		private bool valid = true;

		private Control m_parent;
		private Util util;
		private IRMFields m_fields;
		private ClientSession m_session;
		private IRMComponent m_irm;
		private long m_nFlPlnCmpntID;

		private System.Windows.Forms.Button btnCancel;
		private System.Windows.Forms.Button btnOk;
		private System.Windows.Forms.Label label1;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		#endregion
	}
}
