<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
				xmlns:fo="http://www.w3.org/1999/XSL/Format"
				xmlns:exsl="http://exslt.org/common"
				extension-element-prefixes="exsl"
				version="1.0" >
	
	<xsl:template match="/">

		<fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format">

			<!-- defines page layout -->
			<fo:layout-master-set>
				<fo:simple-page-master master-name="simple" page-width="27.9cm" page-height="21cm" margin-top="0.5cm" margin-bottom="0.5cm" margin-left="0.5cm" margin-right="0.5cm">
					<fo:region-before extent="0.5cm"/>
					<fo:region-body margin-top="0.5cm" margin-bottom="0.5cm"/>
				</fo:simple-page-master>
			</fo:layout-master-set>


			<!-- defines the content -->
			<fo:page-sequence master-reference="simple">
				<fo:static-content flow-name="xsl-region-before" font-size="8pt">
					<fo:block  text-align="right">
					<fo:page-number/>
					<fo:inline>/</fo:inline>
					<fo:page-number-citation ref-id="terminator"/>
					</fo:block>
				</fo:static-content>

				<fo:flow flow-name="xsl-region-body" font-family="Helvetica">
					<fo:block font-size="14pt" font-weight="bold" line-height="26pt" space-after.optimum="12pt" text-align="center">
					File Plan Component Report
					</fo:block>

					<xsl:variable name="numOfDefinitions" select="count(exsl:node-set($config)/List/Item)"/>				
					<fo:table font-size="10pt">
						
						<xsl:variable name="width" select="15 div ($numOfDefinitions + 1)"/>
						<fo:table-column column-width="9.5cm" />
						<fo:table-column column-width="2.5cm" />
						
						<xsl:for-each select="exsl:node-set($config)/List/Item">
						<fo:table-column>
							<xsl:attribute name="column-width">
								<xsl:value-of select="$width"/>
								<xsl:text>cm</xsl:text> 
							</xsl:attribute>
						</fo:table-column>
						</xsl:for-each>

						<fo:table-column>
							<xsl:attribute name="column-width">
								<xsl:value-of select="$width"/>
								<xsl:text>cm</xsl:text> 
							</xsl:attribute>
						</fo:table-column>
						
						<fo:table-header font-weight="bold" color="white" background-color="gray" padding="3pt">
							<fo:table-row >
								<fo:table-cell>
									<fo:block>
									File Plan Component Full Path
									</fo:block>
								</fo:table-cell>

								<fo:table-cell>
									<fo:block >
									Definition Name
									</fo:block>
								</fo:table-cell>

								<xsl:for-each select="exsl:node-set($config)/List/Item">
								<fo:table-cell>
									<fo:block >
									Number of Descendant <xsl:value-of select="@FlPlnCmpntDefNm"/>
									</fo:block>
								</fo:table-cell>
								</xsl:for-each>

								<fo:table-cell>
									<fo:block >
									Number of All Descendant
									</fo:block>
								</fo:table-cell>

							</fo:table-row>													
						</fo:table-header>
						
						<fo:table-body >
							<xsl:for-each select="//Data/List/Item">
							<xsl:variable name="pos" select="position()" />										
							
									<xsl:element name="fo:table-row">
										
										<xsl:if test="($pos mod 2) = 1">
											<xsl:attribute name="background-color">white</xsl:attribute>
										</xsl:if>
										<xsl:if test="($pos mod 2) = 0">
											<xsl:attribute name="background-color">silver</xsl:attribute>
										</xsl:if>
										<xsl:attribute name="padding">3pt</xsl:attribute>
										
										<fo:table-cell>
											<fo:block text-align="start">
												<xsl:value-of select="@NameFullPath" />		 
											</fo:block>
										</fo:table-cell>
										

										<fo:table-cell>
											<fo:block text-align="start">
												<xsl:value-of select="@TS_FlPlnCmpntDef.FlPlnCmpntDefNm" />												
											</fo:block>
										</fo:table-cell>

										<xsl:variable name="curNameFullPath" select="@NameFullPath"/>

										<xsl:variable name="dataList" select="//Data/List"/>
										<xsl:for-each select="exsl:node-set($config)/List/Item">
											<xsl:variable name="curDefName" select="@FlPlnCmpntDefNm"/>

											<fo:table-cell>

												<fo:block text-align="start">
													<xsl:value-of select="count($dataList/Item[starts-with(@NameFullPath, $curNameFullPath) and @TS_FlPlnCmpntDef.FlPlnCmpntDefNm = $curDefName and (@NameFullPath != $curNameFullPath)] )" />
												</fo:block>
											</fo:table-cell>
										</xsl:for-each>

										<fo:table-cell>
											<fo:block text-align="start">
												<xsl:value-of select="count(//Data/List/Item[starts-with(@NameFullPath, $curNameFullPath) and (@NameFullPath!=$curNameFullPath)] )" />												
											</fo:block>
										</fo:table-cell>

									</xsl:element>
							</xsl:for-each>
				
							<!-- Total -->
							<fo:table-row padding="3pt">
								
								<xsl:element name="fo:table-cell">
									<xsl:attribute name="number-columns-spanned">
										<xsl:value-of select="$numOfDefinitions + 3"/>
									</xsl:attribute>
									<xsl:attribute name="border-top" >solid black</xsl:attribute>
									<xsl:attribute name="border-bottom" >solid black</xsl:attribute>
									
									<fo:block text-align="start" >Total:<xsl:value-of select="count(/ResultList/Data/List/Item)" /></fo:block>
									
								</xsl:element>
								
							</fo:table-row>
				
						</fo:table-body>
					</fo:table>
				
				<fo:block id="terminator"/>
				</fo:flow>
			</fo:page-sequence>
		</fo:root>
	</xsl:template>
	
	<!-- ======================================================================================================= 
		Configuration 
	
		By default , it prints the numbers of sub-componet of type class, folder, rec and email for each component. 
		
		You can specify the types of file plan component by editing the report layout file(FPCCountsReport.xsl):
			1)	Open report layout FPCCountsReport.xsl with any XML editor
			2)	Go to the bottom of the file
			3)	Specify types of file plan component by putting them in the list inside element <xsl:variable name="config">.
		For example:
		<xsl:variable name="config">
			<List>
				<Item FlPlnCmpntDefNm="class"/>
				<Item FlPlnCmpntDefNm="folder"/>
				<Item FlPlnCmpntDefNm="rec"/>	
				<Item FlPlnCmpntDefNm="email"/>
			</List>
		</xsl:variable>

		It specifies four types of file plan component: class, folder, rec and email.
	
		========================================================================================================
	-->
	
	<xsl:variable name="config">
		<List>
			<Item FlPlnCmpntDefNm="class"/>
			<Item FlPlnCmpntDefNm="folder"/>
			<Item FlPlnCmpntDefNm="rec"/>	
			<Item FlPlnCmpntDefNm="email"/>
		</List>
	</xsl:variable>

</xsl:stylesheet>